###############################################################
# R code
# File: Metropolis-P3.r
#
# Example 18 in the paper by Amendola and Francq (2009)
# One-dimensional Metropolis-Hastings step to simulate from
# the conditional distribution P3.
#
# Load: MCMCpack (used for inverse Gamma distribution)
#
# Reference:
# Amendola, A. and Francq, C. (2009).
#   Concepts and tools for nonlinear time series modelling.
#   MPRA Paper No. 15140. Available at 
#   htttp://mpra.ub.uni-muenchen.de/15140/, and for a more
#   recent version htttp://mpra.ub.uni-muenchen.de/16668/
################################################################
p3 <- function (gam,beta,sig2,y,n) 
{if (gam<=0) return(-Inf)
Zt       <- matrix(ncol = n, nrow = 4); 
Gt       <- 1/(1+exp(-gam*y));
Zt[1,]   <- 1-Gt; 
Zt[2,]   <- y*(1-Gt); 
Zt[3,]   <- Gt; 
Zt[4,]   <- y*Gt;
eps      <- c(rep(0,n)); 
eps[2:n] <- y[2:n]-beta%*%Zt[1:4,1:(n-1)]-gam-sum(eps^2)/(2*sig2); 
}

gibbsMH  <- function (y,nbatch=1000,theta.init=0.1,tun=1) {
n        <- length(y);
eps      <- c(rep(0,n)); 
Zt       <- matrix(nrow=4,ncol=n);
mat      <- matrix(nrow=6,ncol=nbatch); 
mat[,1]  <- theta.init;

for (i in 2:nbatch) { 
  Gt     <- 1/(1+exp(-mat[5,i-1]*y));
  Zt[1,] <- 1-Gt; 
  Zt[2,] <- y*(1-Gt); 
  Zt[3,] <- Gt; 
  Zt[4,] <- y*Gt;
  ZtZ    <- Zt[1:4,1:(n-1)]%*%t(Zt[1:4,1:(n-1)]);
  Zty    <- {c(sum(y[2:n]*Zt[1,1:(n-1)]),sum(y[2:n]*Zt[2,1:(n-1)]),sum(y[2:n]*Zt[3,1:(n-1)]),sum(y[2:n]*Zt[4,1:(n-1)]))}
  bhat       <- solve(ZtZ,Zty); 
  Sigmaninv  <- ZtZ/mat[6,i-1];
  Sigma      <- solve(diag(rep(1,4))+Sigmaninv);
  mu         <- Sigma%*%(Sigmaninv%*%bhat);
  mat[1:4,i] <- mvrnorm(n=1,as.vector(mu),Sigma);
  eps[2:n]   <- y[2:n]-bhat%*%Zt[1:4,1:(n-1)];
  mat[6,i]   <- rinvgamma(1,shape=1+(n-1)/2,scale=1+sum(eps^2)/2);
  gamstar    <- rnorm(1,mean=mat[5,i-1],sd=tun);
  unif       <- runif(1,0,1);
  lognum     <- p3(gamstar,bhat,mat[6,i],y,n);
  logden     <- p3(mat[5,i-1],bhat,mat[6,i],y,n);
  
  if(lognum==-Inf) ratio<-0 else {if (logden==-Inf) ratio<-1 else ratio<-exp(lognum-logden)}
  mat[5,i]  <- if(unif <= ratio) gamstar else mat[5,i-1]}
  t(mat)
}
